#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for iosxr_acl_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: iosxr_acl_interfaces
short_description: Resource module to configure ACL interfaces.
description:
- This module manages adding and removing Access Control Lists (ACLs) from interfaces
  on devices running IOS-XR software.
version_added: 1.0.0
author: Nilashish Chakraborty (@NilashishC)
options:
  config:
    description: A dictionary of ACL options for interfaces.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Name/Identifier for the interface
        type: str
        required: true
      access_groups:
        type: list
        elements: dict
        description:
        - Specifies ACLs attached to the interfaces.
        suboptions:
          afi:
            description:
            - Specifies the AFI for the ACL(s) to be configured on this interface.
            type: str
            choices:
            - ipv4
            - ipv6
            required: true
          acls:
            type: list
            description:
            - Specifies the ACLs for the provided AFI.
            elements: dict
            suboptions:
              name:
                description:
                - Specifies the name of the IPv4/IPv6 ACL for the interface.
                type: str
                required: true
              direction:
                description:
                - Specifies the direction of packets that the ACL will be applied
                  on.
                type: str
                choices:
                - in
                - out
                required: true
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the IOS-XR device
      by executing the command B(show running-config interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
    - The state the configuration should be left in.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - gathered
    - parsed
    - rendered
    default: merged

"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:22:32.911 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
# !

- name: Merge the provided configuration with the existing running configuration
  cisco.iosxr.iosxr_acl_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        access_groups:
          - afi: ipv4
            acls:
              - name: acl_1
                direction: in
              - name: acl_2
                direction: out
          - afi: ipv6
            acls:
              - name: acl6_1
                direction: in
              - name: acl6_2
                direction: out
      - name: GigabitEthernet0/0/0/1
        access_groups:
          - afi: ipv4
            acls:
              - name: acl_1
                direction: out
    state: merged

# After state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:27:49.378 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !

# Using merged to update interface ACL configuration

# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:27:49.378 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !
#

- name: Update acl_interfaces configuration using merged
  cisco.iosxr.iosxr_acl_interfaces:
    config:
      - name: GigabitEthernet0/0/0/1
        access_groups:
          - afi: ipv4
            acls:
              - name: acl_2
                direction: out
              - name: acl_1
                direction: in
    state: merged

# After state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:27:49.378 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
# !
#

# Using replaced

# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !

- name: >-
    Replace device configurations of listed interface with provided
    configurations
  cisco.iosxr.iosxr_acl_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        access_groups:
          - afi: ipv6
            acls:
              - name: acl6_3
                direction: in
    state: replaced

# After state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv6 access-group acl6_3 ingress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !
#

# Using overridden

# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !
#

- name: Overridde all interface ACL configuration with provided configuration
  cisco.iosxr.iosxr_acl_interfaces:
    config:
      - name: GigabitEthernet0/0/0/1
        access_groups:
          - afi: ipv4
            acls:
              - name: acl_2
                direction: in
          - afi: ipv6
            acls:
              - name: acl6_3
                direction: out
    state: overridden

# After state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_2 ingress
#  ipv6 access-group acl6_3 egress
# !
#

# Using 'deleted' to delete all ACL attributes of a single interface

# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !
#

- name: Delete all ACL attributes of GigabitEthernet0/0/0/1
  cisco.iosxr.iosxr_acl_interfaces:
    config:
      - name: GigabitEthernet0/0/0/1
    state: deleted

# After state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
#

# Using 'deleted' to remove all ACLs attached to all the interfaces in the device

# Before state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !
#

- name: Delete all ACL interfaces configuration from the device
  cisco.iosxr.iosxr_acl_interfaces:
    state: deleted

# After state:
# -------------
#
# RP/0/RP0/CPU0:ios#sh running-config interface
# Wed Jan 15 12:34:56.689 UTC
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
#

# Using parsed

# parsed.cfg
# ------------
#
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  shutdown
#  ipv4 access-group acl_1 ingress
#  ipv4 access-group acl_2 egress
#  ipv6 access-group acl6_1 ingress
#  ipv6 access-group acl6_2 egress
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
#  ipv4 access-group acl_1 egress
# !

# - name: Convert ACL interfaces config to argspec without connecting to the appliance
#   cisco.iosxr.iosxr_acl_interfaces:
#     running_config: "{{ lookup('file', './parsed.cfg') }}"
#     state: parsed


# Task Output (redacted)
# -----------------------

# "parsed": [
#        {
#            "name": "MgmtEth0/RP0/CPU0/0"
#        },
#        {
#            "access_groups": [
#                {
#                    "acls": [
#                        {
#                            "direction": "in",
#                            "name": "acl_1"
#                        },
#                        {
#                            "direction": "out",
#                            "name": "acl_2"
#                        }
#                    ],
#                    "afi": "ipv4"
#                },
#                {
#                    "acls": [
#                        {
#                            "direction": "in",
#                            "name": "acl6_1"
#                        },
#                        {
#                            "direction": "out",
#                            "name": "acl6_2"
#                        }
#                    ],
#                    "afi": "ipv6"
#                }
#            ],
#            "name": "GigabitEthernet0/0/0/0"
#        },
#        {
#            "access_groups": [
#                {
#                    "acls": [
#                        {
#                            "direction": "out",
#                            "name": "acl_1"
#                        }
#                    ],
#                    "afi": "ipv4"
#                }
#            ],
#            "name": "GigabitEthernet0/0/0/1"
#        }
#    ]
# }


# Using gathered

- name: Gather ACL interfaces facts using gathered state
  cisco.iosxr.iosxr_acl_interfaces:
    state: gathered


# Task Output (redacted)
# -----------------------
#
# "gathered": [
#   {
#      "name": "MgmtEth0/RP0/CPU0/0"
#   },
#   {
#      "access_groups": [
#          {
#              "acls": [
#                  {
#                      "direction": "in",
#                      "name": "acl_1"
#                  },
#                  {
#                      "direction": "out",
#                      "name": "acl_2"
#                  }
#              ],
#              "afi": "ipv4"
#          }
#      "name": "GigabitEthernet0/0/0/0"
#  },
#  {
#      "access_groups": [
#          {
#              "acls": [
#                  {
#                      "direction": "in",
#                      "name": "acl6_1"
#                  }
#              ],
#              "afi": "ipv6"
#          }
#       "name": "GigabitEthernet0/0/0/1"
#   }
# ]


# Using rendered

- name: Render platform specific commands from task input using rendered state
  cisco.iosxr.iosxr_acl_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        access_groups:
          - afi: ipv4
            acls:
              - name: acl_1
                direction: in
              - name: acl_2
                direction: out
    state: rendered

# Task Output (redacted)
# -----------------------

# "rendered": [
#     "interface GigabitEthernet0/0/0/0",
#     "ipv4 access-group acl_1 ingress",
#     "ipv4 access-group acl_2 egress"
# ]
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample:
    - "interface GigabitEthernet0/0/0/1"
    - "ipv4 access-group acl_1 ingress"
    - "ipv4 access-group acl_2 egress"
    - "ipv6 access-group acl6_1 ingress"
    - "interface GigabitEthernet0/0/0/2"
    - "no ipv4 access-group acl_3 ingress"
    - "ipv4 access-group acl_4 egress"
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.acl_interfaces.acl_interfaces import (
    Acl_interfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.acl_interfaces.acl_interfaces import (
    Acl_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=Acl_interfacesArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    result = Acl_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_aks_info

version_added: "0.1.2"

short_description: Get Azure Kubernetes Service facts

description:
    - Get facts for a specific Azure Kubernetes Service or all Azure Kubernetes Services.

options:
    name:
        description:
            - Limit results to a specific resource group.
        type: str
    resource_group:
        description:
            - The resource group to search for the desired Azure Kubernetes Service
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
    show_kubeconfig:
        description:
            - Show kubeconfig of the AKS cluster.
            - Note the operation will cost more network overhead, not recommended when listing AKS.
            - I(show_kubeconfig=monitoring) to lists the cluster monitoring user credentials of a managed cluster.
            - I(show_kubeconfig=admin) to lists the cluster admin credentials of a managed cluster.
            - I(show_kubeconfig=user) to lists the cluster user credentials of a managed cluster.
        type: str
        choices:
            - user
            - admin
            - monitoring

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yuwei Zhou (@yuwzho)
'''

EXAMPLES = '''
- name: Get facts for one Azure Kubernetes Service
  azure_rm_aks_info:
    name: Testing
    resource_group: myResourceGroup

- name: Get facts for all Azure Kubernetes Services
  azure_rm_aks_info:

- name: Get facts by tags
  azure_rm_aks_info:
    tags:
      - testing
'''

RETURN = '''
azure_aks:
    description: List of Azure Kubernetes Service dicts.
    returned: always
    type: list
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.exceptions import HttpResponseError
except Exception:
    # handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'managedClusters'


class AzureRMManagedClusterInfo(AzureRMModuleBase):
    """Utility class to get Azure Kubernetes Service facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str'),
            show_kubeconfig=dict(type='str', choices=['user', 'admin', 'monitoring']),
        )

        self.results = dict(
            changed=False,
            aks=[],
            available_versions=[]
        )

        self.name = None
        self.resource_group = None
        self.tags = None
        self.show_kubeconfig = None

        super(AzureRMManagedClusterInfo, self).__init__(
            derived_arg_spec=self.module_args,
            supports_check_mode=True,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_aks_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_aks_facts' module has been renamed to 'azure_rm_aks_info'", version=(2.9, ))

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name is not None and self.resource_group is not None:
            self.results['aks'] = self.get_item()
        elif self.resource_group is not None:
            self.results['aks'] = self.list_by_resourcegroup()
        else:
            self.results['aks'] = self.list_items()

        return self.results

    def get_item(self):
        """Get a single Azure Kubernetes Service"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.managedcluster_client.managed_clusters.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass

        if item and self.has_tags(item.tags, self.tags):
            result = [self.serialize_obj(item, AZURE_OBJECT_CLASS)]
            if self.show_kubeconfig:
                result[0]['kube_config'] = self.get_aks_kubeconfig(self.resource_group, self.name)

        return result

    def list_by_resourcegroup(self):
        """Get all Azure Kubernetes Services"""

        self.log('List all Azure Kubernetes Services under resource group')

        try:
            response = self.managedcluster_client.managed_clusters.list_by_resource_group(self.resource_group)
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                item_dict = self.serialize_obj(item, AZURE_OBJECT_CLASS)
                if self.show_kubeconfig:
                    item_dict['kube_config'] = self.get_aks_kubeconfig(self.resource_group, item.name)
                results.append(item_dict)

        return results

    def list_items(self):
        """Get all Azure Kubernetes Services"""

        self.log('List all Azure Kubernetes Services')

        try:
            response = self.managedcluster_client.managed_clusters.list()
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                item_dict = self.serialize_obj(item, AZURE_OBJECT_CLASS)
                if self.show_kubeconfig:
                    item_dict['kube_config'] = self.get_aks_kubeconfig(item.resource_group, item.name)
                results.append(item_dict)

        return results

    def get_aks_kubeconfig(self, resource_group, name):
        '''
        Gets kubeconfig for the specified AKS instance.

        :return: AKS instance kubeconfig
        '''
        if self.show_kubeconfig == 'user':
            try:
                access_profile = self.managedcluster_client.managed_clusters.list_cluster_user_credentials(self.resource_group, self.name)
            except HttpResponseError as ec:
                self.log("Lists the cluster user credentials of a managed cluster Failed, Exception as {0}".format(ec))
                return []
            return [item.value.decode('utf-8') for item in access_profile.kubeconfigs]
        elif self.show_kubeconfig == 'admin':
            try:
                access_profile = self.managedcluster_client.managed_clusters.list_cluster_admin_credentials(self.resource_group, self.name)
            except HttpResponseError as ec:
                self.log("Lists the cluster admin credentials of a managed cluster Failed, Exception as {0}".format(ec))
                return []
            return [item.value.decode('utf-8') for item in access_profile.kubeconfigs]
        elif self.show_kubeconfig == 'monitoring':
            try:
                access_profile = self.managedcluster_client.managed_clusters.list_cluster_monitoring_user_credentials(self.resource_group, self.name)
            except HttpResponseError as ec:
                self.log("Lists the cluster monitoring credentials of a managed cluster Failed, Exception as {0}".format(ec))
                return []
            return [item.value.decode('utf-8') for item in access_profile.kubeconfigs]
        else:
            return []


def main():
    """Main module execution code path"""

    AzureRMManagedClusterInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2024, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
from ansible.plugins.action import ActionBase

try:
    from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
        AnsibleArgSpecValidator,
    )
except ImportError:
    ANSIBLE_UTILS_IS_INSTALLED = False
else:
    ANSIBLE_UTILS_IS_INSTALLED = True
from ansible.errors import AnsibleActionFail
from ansible_collections.cisco.ise.plugins.plugin_utils.personas_utils import Node

argument_spec = dict(
    primary_ip=dict(type="str", required=True),
    primary_username=dict(type="str", required=True),
    primary_password=dict(type="str", required=True),
    name=dict(type="str", required=True),
    ip=dict(type="str", required=True),
    hostname=dict(type="str", required=True),
    username=dict(type="str", required=True),
    password=dict(type="str", required=True),
    ise_verify=dict(type="bool", default=True),
    ise_version=dict(type="str", default="3.1.0"),
    ise_wait_on_rate_limit=dict(type="bool", default=True),
)

required_if = []
required_one_of = []
mutually_exclusive = []
required_together = []


class ActionModule(ActionBase):
    def __init__(self, *args, **kwargs):
        if not ANSIBLE_UTILS_IS_INSTALLED:
            raise AnsibleActionFail(
                "ansible.utils is not installed. Execute 'ansible-galaxy collection install ansible.utils'"
            )
        super(ActionModule, self).__init__(*args, **kwargs)
        self._supports_async = False
        self._supports_check_mode = False
        self._result = None

    # Checks the supplied parameters against the argument spec for this module
    def _check_argspec(self):
        aav = AnsibleArgSpecValidator(
            data=self._task.args,
            schema=dict(argument_spec=argument_spec),
            schema_format="argspec",
            schema_conditionals=dict(
                required_if=required_if,
                required_one_of=required_one_of,
                mutually_exclusive=mutually_exclusive,
                required_together=required_together,
            ),
            name=self._task.action,
        )
        valid, errors, self._task.args = aav.validate()
        if not valid:
            raise AnsibleActionFail(errors)

    def run(self, tmp=None, task_vars=None):
        self._task.diff = False
        self._result = super(ActionModule, self).run(tmp, task_vars)
        self._result["changed"] = False
        self._check_argspec()

        primary_node = Node(dict(ip=self._task.args.get("primary_ip"),
                                 username=self._task.args.get("primary_username"),
                                 password=self._task.args.get("primary_password"),
                                 )
                            )

        this_node = Node(dict(name=self._task.args.get("name"),
                              ip=self._task.args.get("ip"),
                              hostname=self._task.args.get("hostname"),
                              username=self._task.args.get("username"),
                              password=self._task.args.get("password"),
                              )
                         )

        this_node.import_certificate_into_primary(primary_node)

        response = "The certificate for {hostname} was exported successfully to the primary node".format(hostname=this_node.hostname)

        self._result.update(dict(ise_response=response))
        return self._result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: maps_import_start_v1
short_description: Resource module for Maps Import Start V1
description:
- Manage operation create of the resource Maps Import Start V1.
- >
   Initiates a map archive import of a tar.gz file. The archive must consist of one xmlDir/MapsImportExport.xml map
   descriptor file, and 1 or more images for the map areas nested under /images folder.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options: {}
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites ImportMapArchiveStartImportV1
  description: Complete reference of the ImportMapArchiveStartImportV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!import-map-archive-start-import
notes:
  - SDK Method used are
    sites.Sites.import_map_archive_start_import_v1,

  - Paths used are
    post /dna/intent/api/v1/maps/import/start,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.maps_import_start_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: str
  sample: >
    "'string'"
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_transit_gateway_vpc_attachment
short_description: Create and delete AWS Transit Gateway VPC attachments
version_added: 4.0.0
version_added_collection: community.aws
description:
  - Creates, Deletes and Updates AWS Transit Gateway VPC Attachments.
options:
  transit_gateway:
    description:
      - The ID of the Transit Gateway that the attachment belongs to.
      - When creating a new attachment, O(transit_gateway) must be provided.
      - At least one of O(name), O(transit_gateway) and O(id) must be provided.
      - O(transit_gateway) is an immutable setting and can not be updated on an
        existing attachment.
    type: str
    required: false
    aliases: ["transit_gateway_id"]
  id:
    description:
      - The ID of the Transit Gateway Attachment.
      - When O(id) is not set, a search using O(transit_gateway) and O(name) will be
        performed. If multiple results are returned, the module will fail.
      - At least one of O(name), O(transit_gateway) and O(id) must be provided.
    type: str
    required: false
    aliases: ["attachment_id"]
  name:
    description:
      - The V(Name) tag of the Transit Gateway attachment.
      - Providing both O(id) and O(name) will set the V(Name) tag on an existing
        attachment the matching O(id).
      - Setting the V(Name) tag in O(tags) will also result in the V(Name) tag being
        updated.
      - At least one of O(name), O(transit_gateway) and O(id) must be provided.
    type: str
    required: false
  state:
    description:
      - Create or remove the Transit Gateway attachment.
    type: str
    required: false
    choices: ["present", "absent"]
    default: 'present'
  subnets:
    description:
      - The ID of the subnets in which to create the transit gateway VPC attachment.
      - Required when creating a new attachment.
    type: list
    elements: str
    required: false
  purge_subnets:
    description:
      - If O(purge_subnets=true), existing subnets will be removed from the
        attachment as necessary to match exactly what is defined by O(subnets).
    type: bool
    required: false
    default: true
  dns_support:
    description:
      - Whether DNS support is enabled.
    type: bool
    required: false
  ipv6_support:
    description:
      - Whether IPv6 support is enabled.
    type: bool
    required: false
  appliance_mode_support:
    description:
      - Whether the attachment is configured for appliance mode.
      - When appliance mode is enabled, Transit Gateway, using 4-tuples of an
        IP packet, selects a single Transit Gateway ENI in the Appliance VPC
        for the life of a flow to send traffic to.
    type: bool
    required: false
  wait:
    description:
      - Whether to wait for the Transit Gateway attachment to reach the
        C(Available) or C(Deleted) state before the module returns.
    type: bool
    required: false
    default: true
  wait_timeout:
    description:
      - Maximum time, in seconds, to wait for the Transit Gateway attachment
        to reach the expected state.
      - Defaults to 600 seconds.
    type: int
    default: 600
    required: false
author:
  - Mark Chappell (@tremble)
  - Alina Buzachis (@alinabuzachis)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create a Transit Gateway attachment
  amazon.aws.ec2_transit_gateway_vpc_attachment:
    state: "present"
    transit_gateway: "tgw-123456789abcdef01"
    name: "AnsibleTest-1"
    subnets:
      - "subnet-00000000000000000"
      - "subnet-11111111111111111"
      - "subnet-22222222222222222"
    ipv6_support: true
    purge_subnets: true
    dns_support: true
    appliance_mode_support: true
    tags:
      TestTag: "changed data in Test Tag"

- name: Set sub options on a Transit Gateway attachment
  amazon.aws.ec2_transit_gateway_vpc_attachment:
    state: "present"
    id: "tgw-attach-0c0c5fd0b0f01d1c9"
    name: "AnsibleTest-1"
    ipv6_support: true
    purge_subnets: false
    dns_support: false
    appliance_mode_support: true

- name: Delete the transit gateway
  amazon.aws.ec2_transit_gateway_vpc_attachment:
    state: "absent"
    id: "tgw-attach-0c0c5fd0b0f01d1c9"
"""

RETURN = r"""
attachments:
  description: The attributes of the Transit Gateway attachments.
  type: list
  elements: dict
  returned: success
  contains:
    creation_time:
      description:
        - An ISO 8601 date time stamp of when the attachment was created.
      type: str
      returned: success
      sample: "2022-03-10T16:40:26+00:00"
    options:
      description:
        - Additional VPC attachment options.
      type: dict
      returned: success
      contains:
        appliance_mode_support:
          description:
            - Indicates whether appliance mode support is enabled.
          type: str
          returned: success
          sample: "enable"
        dns_support:
          description:
            - Indicates whether DNS support is enabled.
          type: str
          returned: success
          sample: "disable"
        ipv6_support:
          description:
            - Indicates whether IPv6 support is disabled.
          type: str
          returned: success
          sample: "disable"
        security_group_referencing_support:
          description:
            - Indicated weather security group referencing support is disabled.
          type: str
          returned: success
          sample: "enable"
    state:
      description:
        - The state of the attachment.
      type: str
      returned: success
      sample: "deleting"
    subnet_ids:
      description:
        - The IDs of the subnets in use by the attachment.
      type: list
      elements: str
      returned: success
      sample: ["subnet-0123456789abcdef0", "subnet-11111111111111111"]
    tags:
      description:
        - A dictionary representing the resource tags.
      type: dict
      returned: success
    transit_gateway_attachment_id:
      description:
        - The ID of the attachment.
      type: str
      returned: success
      sample: "tgw-attach-0c0c5fd0b0f01d1c9"
    transit_gateway_id:
      description:
        - The ID of the transit gateway that the attachment is connected to.
      type: str
      returned: success
      sample: "tgw-0123456789abcdef0"
    vpc_id:
      description:
        - The ID of the VPC that the attachment is connected to.
      type: str
      returned: success
      sample: "vpc-0123456789abcdef0"
    vpc_owner_id:
      description:
        - The ID of the account that the VPC belongs to.
      type: str
      returned: success
      sample: "1234567890122"
"""

from typing import NoReturn

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.transformation import boto3_resource_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transitgateway import TransitGatewayVpcAttachmentManager
from ansible_collections.amazon.aws.plugins.module_utils.transitgateway import find_existing_attachment
from ansible_collections.amazon.aws.plugins.module_utils.transitgateway import get_states
from ansible_collections.amazon.aws.plugins.module_utils.transitgateway import subnets_to_vpc


def handle_vpc_attachments(client, module: AnsibleAWSModule) -> NoReturn:
    """
    Handle the creation, modification, or deletion of VPC attachments
    based on the parameters provided in the Ansible module.

    Args:
        client: The AWS client to interact with EC2 services.
        module: An instance of AnsibleAWSModule.

    Returns:
        NoReturn: The function exits by calling module.exit_json()
                  with the results of the operation.
    """
    attach_id = module.params.get("id", None)
    attachment = None

    if not attach_id:
        filters = {}
        if module.params.get("transit_gateway"):
            filters["transit-gateway-id"] = module.params["transit_gateway"]
        if module.params.get("name"):
            filters["tag:Name"] = module.params["name"]
        if module.params.get("subnets"):
            vpc_id = subnets_to_vpc(client, module, module.params["subnets"])
            filters["vpc-id"] = vpc_id

        # Attachments lurk in a 'deleted' state, for a while, ignore them so we
        # can reuse the names
        filters["state"] = get_states()

        attachment = find_existing_attachment(client, module, filters=filters)
        if attachment:
            attach_id = attachment["TransitGatewayAttachmentId"]
    else:
        attachment = find_existing_attachment(client, module, attachment_id=attach_id)

    manager = TransitGatewayVpcAttachmentManager(client, module, attachment, attachment_id=attach_id)

    if module.params["state"] == "absent":
        manager.delete_attachment()
    else:
        manager.create_or_modify_attachment()

    results = dict(
        changed=manager.changed,
        attachments=[manager.updated],
    )
    if manager.changed:
        results["diff"] = dict(
            before=boto3_resource_to_ansible_dict(manager.existing),
            after=manager.updated,
        )

    module.exit_json(**results)


def main():
    argument_spec = dict(
        state=dict(type="str", required=False, default="present", choices=["absent", "present"]),
        transit_gateway=dict(type="str", required=False, aliases=["transit_gateway_id"]),
        id=dict(type="str", required=False, aliases=["attachment_id"]),
        name=dict(type="str", required=False),
        subnets=dict(type="list", elements="str", required=False),
        purge_subnets=dict(type="bool", required=False, default=True),
        tags=dict(type="dict", required=False, aliases=["resource_tags"]),
        purge_tags=dict(type="bool", required=False, default=True),
        appliance_mode_support=dict(type="bool", required=False),
        dns_support=dict(type="bool", required=False),
        ipv6_support=dict(type="bool", required=False),
        wait=dict(type="bool", required=False, default=True),
        wait_timeout=dict(type="int", default=600, required=False),
    )

    one_of = [
        ["id", "transit_gateway", "name"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=one_of,
    )

    client = module.client("ec2")

    handle_vpc_attachments(client, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: assign_device_to_site
short_description: Resource module for Assign Device To Site
description:
- This module represents an alias of the module assign_device_to_site_v1
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  device:
    description: Assign Device To Site's device.
    elements: dict
    suboptions:
      ip:
        description: Device IP. It can be either IPv4 or IPv6. IPV4 e.g., 10.104.240.64.
          IPV6 e.g., 2001 420 284 2004 4 181 500 183.
        type: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  siteId:
    description: SiteId path parameter. Site Id where device(s) needs to be assigned.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites AssignDevicesToSiteV1
  description: Complete reference of the AssignDevicesToSiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!assign-devices-to-site
notes:
  - SDK Method used are
    sites.Sites.assign_devices_to_site_v1,

  - Paths used are
    post /dna/intent/api/v1/assign-device-to-site/{siteId}/device,
  - It should be noted that this module is an alias of assign_device_to_site_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.assign_device_to_site:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    device:
    - ip: string
    headers: '{{my_headers | from_json}}'
    siteId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright: (c) 2015, Ansible, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment:
    # Common configuration for all AWS services
    # Note: If you're updating MODULES, PLUGINS probably needs updating too.

    # Formatted for Modules
    # - modules don't support 'env'
    MODULES = r"""
options:
  region:
    description:
      - The AWS region to use.
      - For global services such as IAM, Route53 and CloudFront, I(region)
        is ignored.
      - The C(AWS_REGION) or C(EC2_REGION) environment variables may also
        be used.
      - See the Amazon AWS documentation for more information
        U(http://docs.aws.amazon.com/general/latest/gr/rande.html#ec2_region).
      - The C(ec2_region) alias has been deprecated and will be removed in
        a release after 2024-12-01
      - Support for the C(EC2_REGION) environment variable has been
        deprecated and will be removed in a release after 2024-12-01.
    type: str
    aliases: [ aws_region, ec2_region ]
"""

    # Formatted for non-module plugins
    # - modules don't support 'env'
    PLUGINS = r"""
options:
  region:
    description:
      - The AWS region to use.
      - See the Amazon AWS documentation for more information
        U(http://docs.aws.amazon.com/general/latest/gr/rande.html#ec2_region).
    type: str
    aliases: [ aws_region, ec2_region ]
    env:
      - name: AWS_REGION
      - name: EC2_REGION
        deprecated:
          removed_at_date: '2024-12-01'
          collection_name: amazon.aws
          why: 'EC2 in the name implied it was limited to EC2 resources, when it is used for all connections'
          alternatives: AWS_REGION
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: custom_issue_definitions_count_info
short_description: Information module for Custom Issue Definitions Count Info
description:
- This module represents an alias of the module custom_issue_definitions_count_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - >
      Id query parameter. The custom issue definition identifier and unique identifier across the profile.
      Examples id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested)
      id=6bef213c-19ca-4170-8375-b694e251101c&id=19ca-4170-8375-b694e251101c-6bef213c (multiple Id request in the
      query param).
    type: str
  profileId:
    description:
    - >
      ProfileId query parameter. The profile identifier to fetch the profile associated custom issue definitions.
      The default is global. For the custom profile, it is profile UUID. Example
      3fa85f64-5717-4562-b3fc-2c963f66afa6.
    type: str
  name:
    description:
    - Name query parameter. The list of UDI issue names. (Ex."TestUdiIssues").
    type: str
  priority:
    description:
    - >
      Priority query parameter. The Issue priority value, possible values are P1, P2, P3, P4. P1 A critical issue
      that needs immediate attention and can have a wide impact on network operations. P2 A major issue that can
      potentially impact multiple devices or clients. P3 A minor issue that has a localized or minimal impact. P4
      A warning issue that may not be an immediate problem but addressing it can optimize the network performance.
    type: str
  isEnabled:
    description:
    - IsEnabled query parameter. The enable status of the custom issue definition, either true or false.
    type: bool
  severity:
    description:
    - >
      Severity query parameter. The syslog severity level. 0 Emergency 1 Alert, 2 Critical. 3 Error, 4 Warning, 5
      Notice, 6 Info. Examples severity=1&severity=2 (multi value support with & separator).
    type: float
  facility:
    description:
    - Facility query parameter. The syslog facility name.
    type: str
  mnemonic:
    description:
    - Mnemonic query parameter. The syslog mnemonic name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Issues GetTheTotalCustomIssueDefinitionsCountBasedOnTheProvidedFiltersV1
  description: Complete reference of the GetTheTotalCustomIssueDefinitionsCountBasedOnTheProvidedFiltersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-total-custom-issue-definitions-count-based-on-the-provided-filters
notes:
  - SDK Method used are
    issues.Issues.get_the_total_custom_issue_definitions_count_based_on_the_provided_filters_v1,

  - Paths used are
    get /dna/intent/api/v1/customIssueDefinitions/count,
  - It should be noted that this module is an alias of custom_issue_definitions_count_v1_info

"""

EXAMPLES = r"""
- name: Get all Custom Issue Definitions Count Info
  cisco.dnac.custom_issue_definitions_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    profileId: string
    name: string
    priority: string
    isEnabled: True
    severity: 0
    facility: string
    mnemonic: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: cidr_merge
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _need_netaddr,
)


__metaclass__ = type

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

DOCUMENTATION = """
    name: cidr_merge
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: This filter can be used to merge subnets or individual addresses.
    description:
        - This filter can be used to merge subnets or individual addresses into their minimal representation, collapsing
        - overlapping subnets and merging adjacent ones wherever possible.
    options:
        value:
            description:
            - list of subnets or individual address to be merged
            type: list
            elements: str
            required: True
        action:
            description:
            - Action to be performed.example merge,span
            default: merge
            type: str
    notes:
"""

EXAMPLES = r"""
#### examples
- name: cidr_merge with merge action
  ansible.builtin.set_fact:
    value:
      - 192.168.0.0/17
      - 192.168.128.0/17
      - 192.168.128.1
- debug:
    msg: '{{ value|ansible.utils.cidr_merge }}'

# TASK [cidr_merge with merge action] **********************************************************************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "value": [
#             "192.168.0.0/17",
#             "192.168.128.0/17",
#             "192.168.128.1"
#         ]
#     },
#     "changed": false
# }
# TASK [debug] *********************************************************************************************************
# ok: [loalhost] => {
#     "msg": [
#         "192.168.0.0/16"
#     ]
# }

- name: Cidr_merge with span.
  ansible.builtin.set_fact:
    value:
      - 192.168.1.1
      - 192.168.1.2
      - 192.168.1.3
      - 192.168.1.4
- debug:
    msg: '{{ value|ansible.utils.cidr_merge(''span'') }}'

# TASK [Cidr_merge with span.] ********************************************************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "value": [
#             "192.168.1.1",
#             "192.168.1.2",
#             "192.168.1.3",
#             "192.168.1.4"
#         ]
#     },
#     "changed": false
# }
#
# TASK [debug] ************************************************************************************
# ok: [localhost] => {
#     "msg": "192.168.1.0/29"
# }
"""

RETURN = """
  data:
    type: raw
    description:
      - Returns a minified list of subnets or a single subnet that spans all of the inputs.
"""


@pass_environment
def _cidr_merge(*args, **kwargs):
    """Convert the given data from json to xml."""
    keys = ["value", "action"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="cidr_merge")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return cidr_merge(**updated_data)


def cidr_merge(value, action="merge"):
    if not hasattr(value, "__iter__"):
        raise AnsibleFilterError("cidr_merge: expected iterable, got " + repr(value))

    if action == "merge":
        try:
            return [str(ip) for ip in netaddr.cidr_merge(value)]
        except Exception as e:
            raise AnsibleFilterError("cidr_merge: error in netaddr:\n%s" % e)

    elif action == "span":
        # spanning_cidr needs at least two values
        if len(value) == 0:
            return None
        elif len(value) == 1:
            try:
                return str(netaddr.IPNetwork(value[0]))
            except Exception as e:
                raise AnsibleFilterError("cidr_merge: error in netaddr:\n%s" % e)
        else:
            try:
                return str(netaddr.spanning_cidr(value))
            except Exception as e:
                raise AnsibleFilterError("cidr_merge: error in netaddr:\n%s" % e)

    else:
        raise AnsibleFilterError("cidr_merge: invalid action '%s'" % action)


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "cidr_merge": _cidr_merge,
    }

    def filters(self):
        if HAS_NETADDR:
            return self.filter_map
        else:
            # Need to install python's netaddr for these filters to work
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_fabric_scheduler
short_description: This module creates ACI schedulers (trig:SchedP)
description:
- With the module you can create schedule policies that can be a shell, one-time execution or recurring.
options:
  name:
    description:
    - The name of the Scheduler.
    type: str
    aliases: [ scheduler_name ]
  description:
    description:
    - Description for the Scheduler.
    type: str
    aliases: [ descr ]
  recurring:
    description:
    - If you want to make the scheduler a recurring operation, it should be set C(True) and for a one-time execution it should be C(False).
    - For a shell, just exclude this option from the task.
    type: bool
  windowname:
    description:
    - The name of the schedule window.
    - This is mandatory for the child class object B(trig:AbsWinddowP)
    type: str
  concurCap:
    description:
    - The amount of devices that can be executed on at a time.
    type: int
  maxTime:
    description:
    - The maximum amount of time a process can be executed.
    type: str
  date:
    description:
    - The date and time that the scheduler will execute.
    type: str
  hour:
    description:
    - The number of hours of execution.
    type: int
  minute:
    description:
    - The number of minutes of execution, used in conjunction with hour.
    type: int
  day:
    description:
    - The number of days when execution will take place.
    type: str
    default: every-day
    choices: [ Monday, Tuesday, Wednesday, Thursday, Friday, Saturday, Sunday, even-day, odd-day, every-day ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    default: present
    choices: [ absent, present, query ]
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(trig:SchedP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Steven Gerhart (@sgerhart)
"""

EXAMPLES = r"""
- name: Simple Scheduler (Empty)
  cisco.aci.aci_fabric_scheduler:
    host: "{{ inventory_hostname }}"
    username: "{{ user }}"
    password: "{{ pass }}"
    validate_certs: false
    name: simpleScheduler
    state: present

- name: Remove Simple Scheduler
  cisco.aci.aci_fabric_scheduler:
    host: "{{ inventory_hostname }}"
    username: "{{ user }}"
    password: "{{ pass }}"
    validate_certs: false
    name: simpleScheduler
    state: absent

- name: One Time Scheduler
  cisco.aci.aci_fabric_scheduler:
    host: "{{ inventory_hostname }}"
    username: "{{ user }}"
    password: "{{ pass }}"
    validate_certs: false
    name: OneTime
    windowname: OneTime
    recurring: false
    concurCap: 20
    date: "2018-11-20T24:00:00"
    state: present

- name: Recurring Scheduler
  cisco.aci.aci_fabric_scheduler:
    host: "{{ inventory_hostname }}"
    username: "{{ user }}"
    password: "{{ pass }}"
    validate_certs: false
    name: Recurring
    windowname: Recurring
    recurring: true
    concurCap: 20
    hour: 13
    minute: 30
    day: Tuesday
    state: present
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["scheduler_name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        windowname=dict(type="str"),
        recurring=dict(type="bool"),
        concurCap=dict(type="int"),  # Number of devices it will run against concurrently
        maxTime=dict(type="str"),  # The amount of minutes a process will be able to run (unlimited or dd:hh:mm:ss)
        date=dict(type="str"),  # The date the process will run YYYY-MM-DDTHH:MM:SS
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        hour=dict(type="int"),
        minute=dict(type="int"),
        day=dict(
            type="str",
            default="every-day",
            choices=["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday", "every-day", "even-day", "odd-day"],
        ),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )

    state = module.params.get("state")
    name = module.params.get("name")
    windowname = module.params.get("windowname")
    recurring = module.params.get("recurring")
    date = module.params.get("date")
    hour = module.params.get("hour")
    minute = module.params.get("minute")
    maxTime = module.params.get("maxTime")
    concurCap = module.params.get("concurCap")
    day = module.params.get("day")
    description = module.params.get("description")
    name_alias = module.params.get("name_alias")

    child_classes = [
        "trigRecurrWindowP",
        "trigAbsWindowP",
    ]

    if recurring:
        child_configs = [
            dict(
                trigRecurrWindowP=dict(
                    attributes=dict(
                        name=windowname,
                        hour=hour,
                        minute=minute,
                        procCa=maxTime,
                        concurCap=concurCap,
                        day=day,
                    )
                )
            )
        ]
    elif recurring is False:
        child_configs = [
            dict(
                trigAbsWindowP=dict(
                    attributes=dict(
                        name=windowname,
                        procCap=maxTime,
                        concurCap=concurCap,
                        date=date,
                    )
                )
            )
        ]
    else:
        child_configs = []

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class="trigSchedP",
            aci_rn="fabric/schedp-{0}".format(name),
            target_filter={"name": name},
            module_object=name,
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="trigSchedP",
            class_config=dict(
                name=name,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="trigSchedP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import re

try:
    import botocore
except ImportError:
    pass  # Modules are responsible for handling this.

from ansible.module_utils._text import to_native

from .arn import parse_aws_arn
from .arn import validate_aws_arn
from .botocore import is_boto3_error_code
from .errors import AWSErrorHandler
from .exceptions import AnsibleAWSError
from .retries import AWSRetry
from .tagging import ansible_dict_to_boto3_tag_list
from .transformation import AnsibleAWSResource
from .transformation import AnsibleAWSResourceList
from .transformation import BotoResource
from .transformation import BotoResourceList
from .transformation import boto3_resource_list_to_ansible_dict
from .transformation import boto3_resource_to_ansible_dict


class AnsibleIAMError(AnsibleAWSError):
    pass


class IAMErrorHandler(AWSErrorHandler):
    _CUSTOM_EXCEPTION = AnsibleIAMError

    @classmethod
    def _is_missing(cls):
        return is_boto3_error_code("NoSuchEntity")


@IAMErrorHandler.deletion_error_handler("detach group policy")
@AWSRetry.jittered_backoff()
def detach_iam_group_policy(client, arn, group):
    client.detach_group_policy(PolicyArn=arn, GroupName=group)
    return True


@IAMErrorHandler.deletion_error_handler("detach role policy")
@AWSRetry.jittered_backoff()
def detach_iam_role_policy(client, arn, role):
    client.detach_role_policy(PolicyArn=arn, RoleName=role)
    return True


@IAMErrorHandler.deletion_error_handler("detach user policy")
@AWSRetry.jittered_backoff()
def detach_iam_user_policy(client, arn, user):
    client.detach_user_policy(PolicyArn=arn, UserName=user)
    return True


@AWSRetry.jittered_backoff()
def _get_iam_instance_profiles(client, **kwargs):
    return client.get_instance_profile(**kwargs)["InstanceProfile"]


@AWSRetry.jittered_backoff()
def _list_iam_instance_profiles(client, **kwargs):
    paginator = client.get_paginator("list_instance_profiles")
    return paginator.paginate(**kwargs).build_full_result()["InstanceProfiles"]


@AWSRetry.jittered_backoff()
def _list_iam_instance_profiles_for_role(client, **kwargs):
    paginator = client.get_paginator("list_instance_profiles_for_role")
    return paginator.paginate(**kwargs).build_full_result()["InstanceProfiles"]


@IAMErrorHandler.list_error_handler("list policies for role", [])
@AWSRetry.jittered_backoff()
def list_iam_role_policies(client, role_name):
    paginator = client.get_paginator("list_role_policies")
    return paginator.paginate(RoleName=role_name).build_full_result()["PolicyNames"]


@IAMErrorHandler.list_error_handler("list policies attached to role", [])
@AWSRetry.jittered_backoff()
def list_iam_role_attached_policies(client, role_name):
    paginator = client.get_paginator("list_attached_role_policies")
    return paginator.paginate(RoleName=role_name).build_full_result()["AttachedPolicies"]


@IAMErrorHandler.list_error_handler("list users", [])
@AWSRetry.jittered_backoff()
def list_iam_users(client, path=None):
    args = {}
    if path is None:
        args = {"PathPrefix": path}
    paginator = client.get_paginator("list_users")
    return paginator.paginate(**args).build_full_result()["Users"]


@IAMErrorHandler.common_error_handler("list all managed policies")
@AWSRetry.jittered_backoff()
def list_iam_managed_policies(client, **kwargs):
    paginator = client.get_paginator("list_policies")
    return paginator.paginate(**kwargs).build_full_result()["Policies"]


list_managed_policies = list_iam_managed_policies


@IAMErrorHandler.list_error_handler("list entities for policy", [])
@AWSRetry.jittered_backoff()
def list_iam_entities_for_policy(client, arn):
    paginator = client.get_paginator("list_entities_for_policy")
    return paginator.paginate(PolicyArn=arn).build_full_result()


@IAMErrorHandler.list_error_handler("list roles", [])
@AWSRetry.jittered_backoff()
def list_iam_roles(client, path=None):
    args = {}
    if path:
        args["PathPrefix"] = path
    paginator = client.get_paginator("list_roles")
    return paginator.paginate(**args).build_full_result()["Roles"]


@IAMErrorHandler.list_error_handler("list mfa devices", [])
@AWSRetry.jittered_backoff()
def list_iam_mfa_devices(client, user=None):
    args = {}
    if user:
        args["UserName"] = user
    paginator = client.get_paginator("list_mfa_devices")
    return paginator.paginate(**args).build_full_result()["MFADevices"]


@IAMErrorHandler.list_error_handler("get role")
@AWSRetry.jittered_backoff()
def get_iam_role(client, name):
    return client.get_role(RoleName=name)["Role"]


@IAMErrorHandler.list_error_handler("get group")
@AWSRetry.jittered_backoff()
def get_iam_group(client, name):
    paginator = client.get_paginator("get_group")
    return paginator.paginate(GroupName=name).build_full_result()


@IAMErrorHandler.list_error_handler("get access keys for user", [])
@AWSRetry.jittered_backoff()
def get_iam_access_keys(client, user):
    results = client.list_access_keys(UserName=user)
    return normalize_iam_access_keys(results.get("AccessKeyMetadata", []))


@IAMErrorHandler.list_error_handler("get user")
@AWSRetry.jittered_backoff()
def get_iam_user(client, user):
    results = client.get_user(UserName=user)
    return normalize_iam_user(results.get("User", []))


def find_iam_managed_policy_by_name(client, name):
    policies = list_iam_managed_policies(client)
    for policy in policies:
        if policy["PolicyName"] == name:
            return policy
    return None


def get_iam_managed_policy_by_name(client, name):
    # get_policy() requires an ARN, and list_policies() doesn't return all fields, so we need to do both :(
    policy = find_iam_managed_policy_by_name(client, name)
    if policy is None:
        return None
    return get_iam_managed_policy_by_arn(client, policy["Arn"])


@IAMErrorHandler.common_error_handler("get policy")
@AWSRetry.jittered_backoff()
def get_iam_managed_policy_by_arn(client, arn):
    policy = client.get_policy(PolicyArn=arn)["Policy"]
    return policy


@IAMErrorHandler.common_error_handler("list policy versions")
@AWSRetry.jittered_backoff()
def list_iam_managed_policy_versions(client, arn):
    return client.list_policy_versions(PolicyArn=arn)["Versions"]


@IAMErrorHandler.common_error_handler("get policy version")
@AWSRetry.jittered_backoff()
def get_iam_managed_policy_version(client, arn, version):
    return client.get_policy_version(PolicyArn=arn, VersionId=version)["PolicyVersion"]


def convert_managed_policy_names_to_arns(client, policy_names):
    if all(validate_aws_arn(policy, service="iam") for policy in policy_names if policy is not None):
        return policy_names
    allpolicies = {}
    policies = list_iam_managed_policies(client)

    for policy in policies:
        allpolicies[policy["PolicyName"]] = policy["Arn"]
        allpolicies[policy["Arn"]] = policy["Arn"]
    try:
        return [allpolicies[policy] for policy in policy_names if policy is not None]
    except KeyError as e:
        raise AnsibleIAMError(message="Failed to find policy by name:" + str(e), exception=e) from e


def get_aws_account_id(module):
    """Given an AnsibleAWSModule instance, get the active AWS account ID"""

    return get_aws_account_info(module)[0]


def get_aws_account_info(module):
    """Given an AnsibleAWSModule instance, return the account information
    (account id and partition) we are currently working on

    get_account_info tries too find out the account that we are working
    on.  It's not guaranteed that this will be easy so we try in
    several different ways.  Giving either IAM or STS privileges to
    the account should be enough to permit this.

    Tries:
    - sts:GetCallerIdentity
    - iam:GetUser
    - sts:DecodeAuthorizationMessage
    """
    account_id = None
    partition = None
    try:
        sts_client = module.client("sts", retry_decorator=AWSRetry.jittered_backoff())
        caller_id = sts_client.get_caller_identity(aws_retry=True)
        account_id = caller_id.get("Account")
        partition = caller_id.get("Arn").split(":")[1]
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError):
        try:
            iam_client = module.client("iam", retry_decorator=AWSRetry.jittered_backoff())
            _arn, partition, _service, _reg, account_id, _resource = iam_client.get_user(aws_retry=True)["User"][
                "Arn"
            ].split(":")
        except is_boto3_error_code("AccessDenied") as e:
            try:
                except_msg = to_native(e.message)
            except AttributeError:
                except_msg = to_native(e)
            result = parse_aws_arn(except_msg)
            if result is None or result["service"] != "iam":
                module.fail_json_aws(
                    e,
                    msg="Failed to get AWS account information, Try allowing sts:GetCallerIdentity or iam:GetUser permissions.",
                )
            account_id = result.get("account_id")
            partition = result.get("partition")
        except (  # pylint: disable=duplicate-except
            botocore.exceptions.BotoCoreError,
            botocore.exceptions.ClientError,
        ) as e:
            module.fail_json_aws(
                e,
                msg="Failed to get AWS account information, Try allowing sts:GetCallerIdentity or iam:GetUser permissions.",
            )

    if account_id is None or partition is None:
        module.fail_json(
            msg="Failed to get AWS account information, Try allowing sts:GetCallerIdentity or iam:GetUser permissions.",
        )

    return (to_native(account_id), to_native(partition))


@IAMErrorHandler.common_error_handler("create instance profile")
@AWSRetry.jittered_backoff()
def create_iam_instance_profile(client, name, path, tags):
    boto3_tags = ansible_dict_to_boto3_tag_list(tags or {})
    path = path or "/"
    result = client.create_instance_profile(InstanceProfileName=name, Path=path, Tags=boto3_tags)
    return result["InstanceProfile"]


@IAMErrorHandler.deletion_error_handler("delete instance profile")
@AWSRetry.jittered_backoff()
def delete_iam_instance_profile(client, name):
    client.delete_instance_profile(InstanceProfileName=name)
    # Error Handler will return False if the resource didn't exist
    return True


@IAMErrorHandler.common_error_handler("add role to instance profile")
@AWSRetry.jittered_backoff()
def add_role_to_iam_instance_profile(client, profile_name, role_name):
    client.add_role_to_instance_profile(InstanceProfileName=profile_name, RoleName=role_name)
    return True


@IAMErrorHandler.deletion_error_handler("remove role from instance profile")
@AWSRetry.jittered_backoff()
def remove_role_from_iam_instance_profile(client, profile_name, role_name):
    client.remove_role_from_instance_profile(InstanceProfileName=profile_name, RoleName=role_name)
    # Error Handler will return False if the resource didn't exist
    return True


@IAMErrorHandler.list_error_handler("list instance profiles", [])
def list_iam_instance_profiles(client, name=None, prefix=None, role=None):
    """
    Returns a list of IAM instance profiles in boto3 format.
    Profiles need to be converted to Ansible format using normalize_iam_instance_profile before being displayed.

    See also: normalize_iam_instance_profile
    """
    if role:
        return _list_iam_instance_profiles_for_role(client, RoleName=role)
    if name:
        # Unlike the others this returns a single result, make this a list with 1 element.
        return [_get_iam_instance_profiles(client, InstanceProfileName=name)]
    if prefix:
        return _list_iam_instance_profiles(client, PathPrefix=prefix)
    return _list_iam_instance_profiles(client)


@IAMErrorHandler.common_error_handler("tag instance profile")
@AWSRetry.jittered_backoff()
def tag_iam_instance_profile(client, name, tags):
    if not tags:
        return
    boto3_tags = ansible_dict_to_boto3_tag_list(tags or {})
    result = client.tag_instance_profile(InstanceProfileName=name, Tags=boto3_tags)


@IAMErrorHandler.common_error_handler("untag instance profile")
@AWSRetry.jittered_backoff()
def untag_iam_instance_profile(client, name, tags):
    if not tags:
        return
    client.untag_instance_profile(InstanceProfileName=name, TagKeys=tags)


@IAMErrorHandler.common_error_handler("tag managed policy")
@AWSRetry.jittered_backoff()
def tag_iam_policy(client, arn, tags):
    if not tags:
        return
    boto3_tags = ansible_dict_to_boto3_tag_list(tags or {})
    client.tag_policy(PolicyArn=arn, Tags=boto3_tags)


@IAMErrorHandler.common_error_handler("untag managed policy")
@AWSRetry.jittered_backoff()
def untag_iam_policy(client, arn, tags):
    if not tags:
        return
    client.untag_policy(PolicyArn=arn, TagKeys=tags)


def _validate_iam_name(resource_type, name=None):
    if name is None:
        return None
    LENGTHS = {"role": 64, "user": 64}
    regex = r"[\w+=,.@-]+"
    max_length = LENGTHS.get(resource_type, 128)
    if len(name) > max_length:
        return f"Length of {resource_type} name may not exceed {max_length}"
    if not re.fullmatch(regex, name):
        return f"{resource_type} name must match pattern {regex}"
    return None


def _validate_iam_path(resource_type, path=None):
    if path is None:
        return None
    regex = r"\/([\w+=,.@-]+\/)*"
    max_length = 512
    if len(path) > max_length:
        return f"Length of {resource_type} path may not exceed {max_length}"
    if not path.endswith("/") or not path.startswith("/"):
        return f"{resource_type} path must begin and end with /"
    if not re.fullmatch(regex, path):
        return f"{resource_type} path must match pattern {regex}"
    return None


def validate_iam_identifiers(resource_type, name=None, path=None):
    name_problem = _validate_iam_name(resource_type, name)
    if name_problem:
        return name_problem
    path_problem = _validate_iam_path(resource_type, path)
    if path_problem:
        return path_problem

    return None


def normalize_iam_mfa_device(device: BotoResource) -> AnsibleAWSResource:
    """Converts IAM MFA Device from the CamelCase boto3 format to the snake_case Ansible format"""
    # MFA Devices don't support Tags (as of 1.34.52)
    return boto3_resource_to_ansible_dict(device)


def normalize_iam_mfa_devices(devices: BotoResourceList) -> AnsibleAWSResourceList:
    """Converts a list of IAM MFA Devices from the CamelCase boto3 format to the snake_case Ansible format"""
    # MFA Devices don't support Tags (as of 1.34.52)
    return boto3_resource_list_to_ansible_dict(devices)


def normalize_iam_user(user: BotoResource) -> AnsibleAWSResource:
    """Converts IAM users from the CamelCase boto3 format to the snake_case Ansible format"""
    return boto3_resource_to_ansible_dict(user)


def normalize_iam_policy(policy: BotoResource) -> AnsibleAWSResource:
    """Converts IAM policies from the CamelCase boto3 format to the snake_case Ansible format"""
    return boto3_resource_to_ansible_dict(policy)


def normalize_iam_group(group: BotoResource) -> AnsibleAWSResource:
    """Converts IAM Groups from the CamelCase boto3 format to the snake_case Ansible format"""
    # Groups don't support Tags (as of 1.34.52)
    return boto3_resource_to_ansible_dict(group, force_tags=False)


def normalize_iam_access_key(access_key: BotoResource) -> AnsibleAWSResource:
    """Converts IAM access keys from the CamelCase boto3 format to the snake_case Ansible format"""
    # Access Keys don't support Tags (as of 1.34.52)
    return boto3_resource_to_ansible_dict(access_key, force_tags=False)


def normalize_iam_access_keys(access_keys: BotoResourceList) -> AnsibleAWSResourceList:
    """Converts a list of IAM access keys from the CamelCase boto3 format to the snake_case Ansible format"""
    # Access Keys don't support Tags (as of 1.34.52)
    if not access_keys:
        return access_keys
    access_keys = boto3_resource_list_to_ansible_dict(access_keys, force_tags=False)
    return sorted(access_keys, key=lambda d: d.get("create_date", None))


def normalize_iam_instance_profile(profile: BotoResource) -> AnsibleAWSResource:
    """
    Converts a boto3 format IAM instance profile into "Ansible" format
    """
    transforms = {"Roles": _normalize_iam_roles}
    transformed_profile = boto3_resource_to_ansible_dict(profile, nested_transforms=transforms)
    return transformed_profile


def normalize_iam_role(role: BotoResource, _v7_compat: bool = False) -> AnsibleAWSResource:
    """
    Converts a boto3 format IAM instance role into "Ansible" format

    _v7_compat is deprecated and will be removed in release after 2026-05-01 DO NOT USE.
    """
    transforms = {"InstanceProfiles": _normalize_iam_instance_profiles}
    ignore_list = ["AssumeRolePolicyDocument"]
    transformed_role = boto3_resource_to_ansible_dict(role, nested_transforms=transforms, ignore_list=ignore_list)
    if _v7_compat and role.get("AssumeRolePolicyDocument"):
        transformed_role["assume_role_policy_document_raw"] = role["AssumeRolePolicyDocument"]
    return transformed_role


def _normalize_iam_instance_profiles(profiles: BotoResourceList) -> AnsibleAWSResourceList:
    if not profiles:
        return profiles
    return [normalize_iam_instance_profile(p) for p in profiles]


def _normalize_iam_roles(roles: BotoResourceList) -> AnsibleAWSResourceList:
    if not roles:
        return roles
    return [normalize_iam_role(r) for r in roles]



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_evpn_global config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.evpn_global import (
    Evpn_globalTemplate,
)


EVPN_GLOBAL_PARENT = "l2vpn evpn"


class Evpn_global(ResourceModule):
    """
    The ios_evpn_global config class
    """

    def __init__(self, module):
        super(Evpn_global, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="evpn_global",
            tmplt=Evpn_globalTemplate(),
        )
        self.parsers = [
            "default_gateway.advertise",
            "flooding_suppression.address_resolution.disable",
            "ip.local_learning.disable",
            "replication_type",
            "route_target.auto.vni",
            "router_id",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self.want
        haved = self.have

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # remove superfluous config for deleted
        if self.state == "deleted":
            if haved:
                self.commands.append("no " + EVPN_GLOBAL_PARENT)
            wantd, haved = {}, {}

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Evpn_global network resource.
        """
        begin = len(self.commands)
        self.compare(parsers=self.parsers, want=want, have=have)
        if len(self.commands) != begin:
            self.commands.insert(begin, EVPN_GLOBAL_PARENT)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sites_time_zone_settings_v1_info
short_description: Information module for Sites Time Zone Settings V1
description:
- Get all Sites Time Zone Settings V1.
- >
   Retrieve time zone settings for a site; `null` values indicate that the setting will be inherited from the parent
   site; empty objects `{}` indicate that the setting is unset at a site.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id path parameter. Site Id.
    type: str
  _inherited:
    description:
    - >
      _inherited query parameter. Include settings explicitly set for this site and settings inherited from sites
      higher in the site hierarchy; when `false`, `null` values indicate that the site inherits that setting from
      the parent site or a site higher in the site hierarchy.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings RetrieveTimeZoneSettingsForASiteV1
  description: Complete reference of the RetrieveTimeZoneSettingsForASiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieve-time-zone-settings-for-a-site
notes:
  - SDK Method used are
    network_settings.NetworkSettings.retrieve_time_zone_settings_for_a_site_v1,

  - Paths used are
    get /dna/intent/api/v1/sites/{id}/timeZoneSettings,

"""

EXAMPLES = r"""
- name: Get all Sites Time Zone Settings V1
  cisco.dnac.sites_time_zone_settings_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    _inherited: True
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "timeZone": {
          "identifier": "string",
          "inheritedSiteId": "string",
          "inheritedSiteName": "string"
        }
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos ntp_global fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.ntp_global.ntp_global import (
    Ntp_globalArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.ntp_global import (
    Ntp_globalTemplate,
)


class Ntp_globalFacts(object):
    """The eos ntp_global facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Ntp_globalArgs.argument_spec

    def get_config(self, connection):
        """Wrapper method for `connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return connection.get("show running-config | section ntp")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Ntp_global network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = {}

        if not data:
            data = self.get_config(connection)

        # parse native config using the Ntp_global template
        ntp_global_parser = Ntp_globalTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        objs = ntp_global_parser.parse()
        if objs:
            if "authentication_keys" in objs:
                objs["authentication_keys"] = sorted(
                    list(objs["authentication_keys"].values()),
                    key=lambda k, sk="id": k[sk],
                )
            if "serve" in objs:
                if "access_lists" in objs["serve"]:
                    objs["serve"]["access_lists"] = sorted(
                        list(objs["serve"]["access_lists"].values()),
                        key=lambda k, sk="afi": k[sk],
                    )
            if "servers" in objs:
                objs["servers"] = sorted(
                    list(objs["servers"].values()),
                    key=lambda k, sk="server": k[sk],
                )
        else:
            objs = {}
        ansible_facts["ansible_network_resources"].pop("ntp_global", None)

        params = utils.remove_empties(
            ntp_global_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["ntp_global"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Vrf_address_family parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Vrf_address_familyTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Vrf_address_familyTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "vrf {{ name }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                },
            },
            "shared": True,
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "address-family {{ afi }} {{ safi }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                        },
                    },
                },
            },
        },
        {
            "name": "export.route_policy",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+export\sroute-policy\s(?P<export_route_policy>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "export route-policy {{ export.route_policy }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "export": {
                                "route_policy": "{{ export_route_policy }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "export.route_target",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+export\sroute-target\s(?P<export_route_target>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "export route-target {{ export.route_target }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "export": {
                                "route_target": "{{ export_route_target }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "export.to.default_vrf.route_policy",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+export\sto\sdefault-vrf\sroute-policy\s(?P<export_to_default_vrf_route_policy>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "export to default-vrf route-policy {{ export.to.default_vrf.route_policy }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "export": {
                                "to": {
                                    "default_vrf": {
                                        "route_policy": "{{ export_to_default_vrf_route_policy }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "export.to.vrf.allow_imported_vpn",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+export\sto\svrf\s(?P<allow_imported_vpn>allow-imported-vpn)
                $""", re.VERBOSE,
            ),
            "setval": "export to vrf allow-imported-vpn",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "export": {
                                "to": {
                                    "vrf": {
                                        "allow_imported_vpn": "{{ true if allow_imported_vpn is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.route_target",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+import\sroute-target\s(?P<import_config_route_target>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "import route-target {{ import_config.route_target }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "import_config": {
                                "route_target": "{{import_config_route_target}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.route_policy",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+import\sroute-policy\s(?P<import_config_route_policy>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "import route-policy {{ import_config.route_policy }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "import_config": {
                                "route_policy": "{{import_config_route_policy}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.from_config.bridge_domain.advertise_as_vpn",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+import\sfrom\sbridge-domain\s(?P<advertise_as_vpn>advertise-as-vpn)
                $""", re.VERBOSE,
            ),
            "setval": "import from bridge-domain advertise-as-vpn",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "import_config": {
                                "from_config": {
                                    "bridge_domain": {
                                        "advertise_as_vpn": "{{ true if advertise_as_vpn is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.from_config.default_vrf.route_policy",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+import\sfrom\sdefault-vrf\sroute-policy\s(?P<import_config_from_config_default_vrf_route_policy>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "import from default-vrf route-policy {{ import_config.from_config.default_vrf.route_policy }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "import_config": {
                                "from_config": {
                                    "default_vrf": {
                                        "route_policy": "{{ import_config_from_config_default_vrf_route_policy }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "import_config.from_config.vrf.advertise_as_vpn",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+import\sfrom\svrf\s(?P<advertise_as_vpn>advertise-as-vpn)
                $""", re.VERBOSE,
            ),
            "setval": "import from vrf advertise-as-vpn",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "import_config": {
                                "from_config": {
                                    "vrf": {
                                        "advertise_as_vpn": "{{ true if advertise_as_vpn is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum.prefix",
            "getval": re.compile(
                r"""
                ^vrf\s(?P<name>\S+)
                (?P<address_families>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                \s+maximum\sprefix\s(?P<maximum_prefix>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "maximum prefix {{ maximum.prefix }}",
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                    "address_families": {
                        '{{"address_families_" + afi + "_" + safi }}': {
                            "afi": "{{ afi}}",
                            "safi": "{{safi}}",
                            "maximum": {
                                "prefix": "{{ maximum_prefix }}",
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_device_controllability_settings_v1_info
short_description: Information module for Network Devices Device Controllability Settings V1
description:
- Get all Network Devices Device Controllability Settings V1.
- >
   Device Controllability is a system-level process on Catalyst Center that enforces state synchronization for some
   device-layer features. Its purpose is to aid in the deployment of required network settings that Catalyst Center
   needs to manage devices. Changes are made on network devices during discovery, when adding a device to Inventory,
   or when assigning a device to a site. If changes are made to any settings that are under the scope of this
   process, these changes are applied to the network devices during the Provision and Update Telemetry Settings
   operations, even if Device Controllability is disabled. The following device settings will be enabled as part of
   Device Controllability when devices are discovered. - SNMP Credentials. - NETCONF Credentials. Subsequent to
   discovery, devices will be added to Inventory. The following device settings will be enabled when devices are
   added to inventory. - Cisco TrustSec CTS Credentials. The following device settings will be enabled when devices
   are assigned to a site. Some of these settings can be defined at a site level under Design > Network Settings >
   Telemetry & Wireless. - Wired Endpoint Data Collection Enablement. - Controller Certificates. - SNMP Trap Server
   Definitions. - Syslog Server Definitions. - Application Visibility. - Application QoS Policy. - Wireless Service
   Assurance WSA. - Wireless Telemetry. - DTLS Ciphersuite. - AP Impersonation. If Device Controllability is
   disabled, Catalyst Center does not configure any of the preceding credentials or settings on devices during
   discovery, at runtime, or during site assignment. However, the telemetry settings and related configuration are
   pushed when the device is provisioned or when the update Telemetry Settings action is performed. Catalyst Center
   identifies and automatically corrects the following telemetry configuration issues on the device. - SWIM
   certificate issue. - IOS WLC NA certificate issue. - PKCS12 certificate issue. - IOS telemetry configuration issu.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design GetDeviceControllabilitySettingsV1
  description: Complete reference of the GetDeviceControllabilitySettingsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-controllability-settings
notes:
  - SDK Method used are
    site_design.SiteDesign.get_device_controllability_settings_v1,

  - Paths used are
    get /dna/intent/api/v1/networkDevices/deviceControllability/settings,

"""

EXAMPLES = r"""
- name: Get all Network Devices Device Controllability Settings V1
  cisco.dnac.network_devices_device_controllability_settings_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "autocorrectTelemetryConfig": true,
        "deviceControllability": true
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_devices_interfaces_query_v2
short_description: Resource module for Network Devices Interfaces Query V2
description:
- Manage operation create of the resource Network Devices Interfaces Query V2.
- >
   This API returns the Interface Stats for the given Device Id. Please refer to the Feature tab for the Request Body
   usage and the API filtering support.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceId:
    description: DeviceId path parameter. Network Device Id.
    type: str
  endTime:
    description: UTC epoch timestamp in milliseconds.
    type: int
  query:
    description: Network Devices Interfaces Query's query.
    suboptions:
      fields:
        description: Required field names, default ALL.
        elements: dict
        type: list
      filters:
        description: Network Devices Interfaces Query's filters.
        elements: dict
        suboptions:
          key:
            description: Name of the field that the filter should be applied to.
            type: str
          operator:
            description: Supported operators are eq,in,like.
            type: str
          value:
            description: Value of the field.
            type: str
        type: list
      page:
        description: Network Devices Interfaces Query's page.
        suboptions:
          limit:
            description: Number of records, Max is 1000.
            type: int
          offset:
            description: Record offset value, default 0.
            type: float
          orderBy:
            description: Network Devices Interfaces Query's orderBy.
            elements: dict
            suboptions:
              name:
                description: Name of the field used to sort.
                type: str
              order:
                description: Possible values asc, des.
                type: str
            type: list
        type: dict
    type: dict
  startTime:
    description: UTC epoch timestamp in milliseconds.
    type: int
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetDeviceInterfaceStatsInfoV2
  description: Complete reference of the GetDeviceInterfaceStatsInfoV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-interface-stats-info-v-2
notes:
  - SDK Method used are
    devices.Devices.get_device_interface_stats_info_v2,

  - Paths used are
    post /dna/intent/api/v2/networkDevices/{deviceId}/interfaces/query,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.network_devices_interfaces_query_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceId: string
    endTime: 0
    query:
      fields:
      - {}
      filters:
      - key: string
        operator: string
        value: string
      page:
        limit: 0
        offset: 0
        orderBy:
        - name: string
          order: string
    startTime: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "totalCount": 0,
      "response": [
        {
          "id": "string",
          "values": {
            "adminStatus": "string",
            "deviceId": "string",
            "duplexConfig": "string",
            "duplexOper": "string",
            "interfaceId": "string",
            "interfaceType": "string",
            "instanceId": "string",
            "ipv4Address": "string",
            "ipv6AddressList": [
              "string"
            ],
            "isL3Interface": "string",
            "isWan": "string",
            "macAddr": "string",
            "mediaType": "string",
            "name": "string",
            "operStatus": "string",
            "peerStackMember": "string",
            "peerStackPort": "string",
            "portChannelId": "string",
            "portMode": "string",
            "portType": "string",
            "description": "string",
            "rxDiscards": "string",
            "rxError": "string",
            "rxRate": "string",
            "rxUtilization": "string",
            "speed": "string",
            "stackPortType": "string",
            "timestamp": "string",
            "txDiscards": "string",
            "txError": "string",
            "txRate": "string",
            "txUtilization": "string",
            "vlanId": "string"
          }
        }
      ],
      "page": {
        "limit": 0,
        "offset": 0,
        "count": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: clients_query_v1
short_description: Resource module for Clients Query V1
description:
- Manage operation create of the resource Clients Query V1.
- >
   Retrieves the list of clients by applying complex filters while also supporting aggregate attributes. For detailed
   information about the usage of the API, please refer to the Open API specification document - https
   //github.com/cisco-en-programmability/catalyst-center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-
   clients1-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Clients Query's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Clients Query's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: int
    type: list
  headers:
    description: Additional headers.
    type: dict
  page:
    description: Clients Query's page.
    suboptions:
      limit:
        description: Limit.
        type: int
      offset:
        description: Offset.
        type: int
      sortBy:
        description: Clients Query's sortBy.
        elements: dict
        suboptions:
          name:
            description: Name.
            type: str
          order:
            description: Order.
            type: str
        type: list
    type: dict
  startTime:
    description: Start Time.
    type: int
  views:
    description: Views.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients RetrievesTheListOfClientsByApplyingComplexFiltersWhileAlsoSupportingAggregateAttributesV1
  description: Complete reference of the RetrievesTheListOfClientsByApplyingComplexFiltersWhileAlsoSupportingAggregateAttributesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-list-of-clients-by-applying-complex-filters-while-also-supporting-aggregate-attributes
notes:
  - SDK Method used are
    clients.Clients.retrieves_the_list_of_clients_by_applying_complex_filters_while_also_supporting_aggregate_attributes_v1,

  - Paths used are
    post /dna/data/api/v1/clients/query,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.clients_query_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - key: string
      operator: string
      value: 0
    headers: '{{my_headers | from_json}}'
    page:
      limit: 0
      offset: 0
      sortBy:
      - name: string
        order: string
    startTime: 0
    views:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "macAddress": "string",
          "type": "string",
          "name": "string",
          "userId": "string",
          "username": "string",
          "ipv4Address": "string",
          "ipv6Addresses": [
            "string"
          ],
          "vendor": "string",
          "osType": "string",
          "osVersion": "string",
          "formFactor": "string",
          "siteHierarchy": "string",
          "siteHierarchyId": "string",
          "siteId": "string",
          "lastUpdatedTime": 0,
          "connectionStatus": "string",
          "tracked": "string",
          "isPrivateMacAddress": true,
          "health": {
            "overallScore": 0,
            "onboardingScore": 0,
            "connectedScore": 0,
            "linkErrorPercentageThreshold": 0,
            "isLinkErrorIncluded": true,
            "rssiThreshold": 0,
            "snrThreshold": 0,
            "isRssiIncluded": true,
            "isSnrIncluded": true
          },
          "traffic": {
            "txBytes": 0,
            "rxBytes": 0,
            "usage": 0,
            "rxPackets": 0,
            "txPackets": 0,
            "rxRate": 0,
            "txRate": 0,
            "rxLinkErrorPercentage": 0,
            "txLinkErrorPercentage": 0,
            "rxRetries": 0,
            "rxRetryPercentage": 0,
            "txDrops": 0,
            "txDropPercentage": 0,
            "dnsRequestCount": 0,
            "dnsResponseCount": 0
          },
          "connectedNetworkDevice": {
            "connectedNetworkDeviceId": "string",
            "connectedNetworkDeviceName": "string",
            "connectedNetworkDeviceManagementIp": "string",
            "connectedNetworkDeviceMac": "string",
            "connectedNetworkDeviceType": "string",
            "interfaceName": "string",
            "interfaceSpeed": 0,
            "duplexMode": "string"
          },
          "connection": {
            "vlanId": "string",
            "sessionDuration": 0,
            "vnId": "string",
            "l2Vn": "string",
            "l3Vn": "string",
            "securityGroupTag": "string",
            "linkSpeed": 0,
            "bridgeVMMode": "string",
            "band": "string",
            "ssid": "string",
            "authType": "string",
            "wlcName": "string",
            "wlcId": "string",
            "apMac": "string",
            "apEthernetMac": "string",
            "apMode": "string",
            "radioId": 0,
            "channel": "string",
            "channelWidth": "string",
            "protocol": "string",
            "protocolCapability": "string",
            "upnId": "string",
            "upnName": "string",
            "upnOwner": "string",
            "upnDuid": "string",
            "rssi": 0,
            "snr": 0,
            "dataRate": 0,
            "isIosAnalyticsCapable": true
          },
          "onboarding": {
            "avgRunDuration": 0,
            "maxRunDuration": 0,
            "avgAssocDuration": 0,
            "maxAssocDuration": 0,
            "avgAuthDuration": 0,
            "maxAuthDuration": 0,
            "avgDhcpDuration": 0,
            "maxDhcpDuration": 0,
            "maxRoamingDuration": 0,
            "aaaServerIp": "string",
            "dhcpServerIp": "string",
            "onboardingTime": 0,
            "authDoneTime": 0,
            "assocDoneTime": 0,
            "dhcpDoneTime": 0,
            "roamingTime": 0,
            "failedRoamingCount": 0,
            "successfulRoamingCount": 0,
            "totalRoamingAttempts": 0,
            "assocFailureReason": "string",
            "aaaFailureReason": "string",
            "dhcpFailureReason": "string",
            "otherFailureReason": "string",
            "latestFailureReason": "string"
          },
          "latency": {
            "video": 0,
            "voice": 0,
            "bestEffort": 0,
            "background": 0
          },
          "aggregateAttributes": [
            {
              "name": "string",
              "function": "string",
              "value": 0
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "offset": 0,
        "count": 0,
        "sortBy": [
          {
            "name": "string",
            "order": "string"
          }
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

try:
    from ansible.errors import AnsibleActionFail
except ImportError:
    ANSIBLE_ERRORS_INSTALLED = False
else:
    ANSIBLE_ERRORS_INSTALLED = True
try:
    import requests
except ImportError:
    REQUESTS_INSTALLED = False
else:
    REQUESTS_INSTALLED = True
import json
import zipfile
import io


class Node(object):
    def __init__(self, node):
        requests.packages.urllib3.disable_warnings()
        self.name = node.get("name")
        self.ip = node.get("ip")
        self.fqdn = node.get("fqdn")
        self.hostname = node.get("hostname")
        self.username = node.get("username")
        self.password = node.get("password")
        self.roles = node.get("roles")
        self.services = node.get("services")

    def __str__(self):
        return "{name} <{ip}>".format(name=self.name, ip=self.ip)

    def __repr__(self):
        return "{name} <{ip}>".format(name=self.name, ip=self.ip)

    def is_standalone(self):
        headers = {'Content-Type': 'application/json'}
        url = "https://{ip}/api/v1/deployment/node/{hostname}".format(ip=self.ip, hostname=self.hostname)
        response = False
        try:
            response = requests.get(url=url, headers=headers, auth=(self.username, self.password), verify=False)
        except Exception as e:
            raise AnsibleActionFail("Couldn't connect, the node might be still initializing, try again in a few minutes. Error received: {e}".format(e=e))
        if not response:
            raise AnsibleActionFail("Couldn't get a valid response from the API. Maybe the node is still initializing, try again in a few minutes.")
        else:
            response = json.loads(response.text).get("response")
            if "Standalone" in response.get("roles"):
                return True
        return False

    def app_server_is_running(self):
        url = "https://{ip}/ers/config/op/systemconfig/iseversion".format(ip=self.ip)
        headers = {'Accept': 'application/json'}
        try:
            response = requests.get(url=url, headers=headers, auth=(self.username, self.password), verify=False)
            # Application Server is down but API Gateway is up
            if response.status_code == 502:
                return False
            # The Application Server is up
            if response.status_code == 200:
                return True
            # Any other case return False
            else:
                return False
        # Both Application Server and API Gateway are down
        except Exception as e:
            return False

    def return_id_of_certificate(self):
        url = "https://{ip}/api/v1/certs/system-certificate/{hostname}".format(ip=self.ip, hostname=self.hostname)
        headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        try:
            response = requests.get(url=url, timeout=15, headers=headers, auth=(self.username, self.password), verify=False)
        except requests.exceptions.ReadTimeout:
            raise AnsibleActionFail("The request timed out. Please verify that the API is enabled on the node.")
        except Exception as e:
            raise AnsibleActionFail(e)
        json_response = json.loads(response.text)
        for item in json_response.get("response"):
            if item.get("friendlyName") == "Default self-signed server certificate":
                return item.get("id")

    def register_to_primary(self, primary):
        headers = {'Content-Type': 'application/json'}
        url = "https://{primary_ip}/api/v1/deployment/node".format(primary_ip=primary.ip)
        data = json.dumps({
            "fqdn": self.fqdn,
            "userName": self.username,
            "password": self.password,
            "allowCertImport": True,
            "roles": self.roles,
            "services": self.services
        })
        try:
            response = requests.post(url=url, timeout=300, auth=(primary.username, primary.password), headers=headers, data=data, verify=False)
        except Exception as e:
            raise AnsibleActionFail(e)
        if not response:
            raise AnsibleActionFail("Failed to receive a valid response from the API. The actual response was: {response}".format(response=response.text))

    def get_roles_services(self):
        headers = {'Content-Type': 'application/json'}
        url = "https://{ip}/api/v1/deployment/node/{hostname}".format(ip=self.ip, hostname=self.hostname)
        response = False
        try:
            response = requests.get(url=url, headers=headers, auth=(self.username, self.password), verify=False)
        except Exception as e:
            raise AnsibleActionFail("Couldn't connect, the node might be still initializing, try again in a few minutes. Error received: {e}".format(e=e))
        if not response:
            raise AnsibleActionFail("Couldn't get a valid response from the API. Maybe the node is still initializing, try again in a few minutes.")
        else:
            response = json.loads(response.text).get("response")
        return response

    def update_roles_services(self):
        headers = {'Content-Type': 'application/json'}
        url = "https://{ip}/api/v1/deployment/node/{hostname}".format(ip=self.ip, hostname=self.hostname)
        data = json.dumps({
            "roles": self.roles,
            "services": self.services
        })
        try:
            response = requests.put(url=url, timeout=300, auth=(self.username, self.password), headers=headers, data=data, verify=False)
        except Exception as e:
            raise AnsibleActionFail(e)
        if not response:
            raise AnsibleActionFail("Failed to receive a valid response from the API. The actual response was: {response}".format(response=response.text))

    def import_certificate_into_primary(self, primary_node):
        cert_id = self.return_id_of_certificate()
        data = json.dumps({"id": cert_id, "export": "CERTIFICATE"})
        url = "https://{ip}/api/v1/certs/system-certificate/export".format(ip=self.ip)
        headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        try:
            response = requests.post(url=url, timeout=15, data=data, headers=headers, auth=(self.username, self.password), verify=False)
        except Exception as e:
            raise AnsibleActionFail(e)

        if not response.status_code == 200:
            raise AnsibleActionFail("Received status code {status_code} when exporting certificate.".format(status_code=str(response.status_code)))

        zf = zipfile.ZipFile(io.BytesIO(response.content), 'r')
        cert_data = zf.read("Defaultselfsignedservercerti.pem")
        data = json.dumps({
            "allowBasicConstraintCAFalse": True,
            "allowOutOfDateCert": False,
            "allowSHA1Certificates": True,
            "trustForCertificateBasedAdminAuth": True,
            "trustForCiscoServicesAuth": True,
            "trustForClientAuth": True,
            "data": cert_data.decode("utf-8"),
            "trustForIseAuth": True,
            "name": self.name,
            "validateCertificateExtensions": True
        })
        url = "https://{primary_ip}/api/v1/certs/trusted-certificate/import".format(primary_ip=primary_node.ip)
        headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
        try:
            response = requests.post(url=url,
                                     timeout=15,
                                     data=data,
                                     headers=headers,
                                     auth=(self.primary_node.username, self.primary_node.password),
                                     verify=False
                                     )
            return_message = json.loads(response.text)["response"]["message"]
        except Exception as e:
            raise AnsibleActionFail(e)

        if not response.status_code == 200:
            if not (return_message == 'Trust certificate was added successfully' or
                    return_message == "Certificates are having same subject, same serial number and they are binary equal. Hence skipping the replace"):
                raise AnsibleActionFail("Unexpected response from API. Received response was {message}".format(message=return_message))

    def promote_to_primary(self):
        headers = {'Content-Type': 'application/json'}
        url = "https://{ip}/api/v1/deployment/primary".format(ip=self.ip)
        try:
            response = requests.post(url=url, headers=headers, auth=(self.username, self.password), verify=False, timeout=60)
            if response.status_code == 200:
                return True
            else:
                raise AnsibleActionFail("Could not update node to PRIMARY")
        except Exception as e:
            raise AnsibleActionFail(e)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_psk_override_v1
short_description: Resource module for Wireless Psk Override V1
description:
- Manage operation create of the resource Wireless Psk Override V1.
- Update/Override passphrase of SSID.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Wireless Psk Override's payload.
    elements: dict
    suboptions:
      passPhrase:
        description: Pass phrase (create/update).
        type: str
      site:
        description: Site name hierarchy (ex Global/aaa/zzz/...).
        type: str
      ssid:
        description: Enterprise ssid name(already created/present).
        type: str
      wlanProfileName:
        description: WLAN Profile Name.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless PSKOverrideV1
  description: Complete reference of the PSKOverrideV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!p-sk-override
notes:
  - SDK Method used are
    wireless.Wireless.psk_override,

  - Paths used are
    post /dna/intent/api/v1/wireless/psk-override,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_psk_override_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - passPhrase: string
      site: string
      ssid: string
      wlanProfileName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionStatusUrl": "string",
      "message": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: backup_tag_info
version_added: 6.0.0
short_description: List tags on AWS Backup resources
description:
    - List tags on AWS backup resources such as backup plan, backup vault, and recovery point.
    - Resources are referenced using ARN.
author:
    - Mandar Vijay Kulkarni (@mandar242)
options:
  resource:
    description:
      - The Amazon Resource Name (ARN) of the backup resource.
    required: true
    type: str

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: List tags on a resource
  amazon.aws.backup_tag_info:
    resource: "{{ backup_resource_arn }}"
"""

RETURN = r"""
tags:
  description: A dict containing the tags on the resource.
  returned: always
  type: dict
"""

from ansible_collections.amazon.aws.plugins.module_utils.backup import get_backup_resource_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule


def main():
    argument_spec = dict(
        resource=dict(required=True, type="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    backup_client = module.client("backup")

    current_tags = get_backup_resource_tags(module, backup_client, module.params["resource"])

    module.exit_json(changed=False, tags=current_tags)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: health_score_definitions_bulk_update
short_description: Resource module for Health Score Definitions Bulk Update
description:
- This module represents an alias of the module health_score_definitions_bulk_update_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  payload:
    description: Health Score Definitions Bulk Update's payload.
    elements: dict
    suboptions:
      id:
        description: Id.
        type: str
      includeForOverallHealth:
        description: Include For Overall Health.
        type: bool
      synchronizeToIssueThreshold:
        description: Synchronize To Issue Threshold.
        type: bool
      thresholdValue:
        description: Threshold Value.
        type: float
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices UpdateHealthScoreDefinitionsV1
  description: Complete reference of the UpdateHealthScoreDefinitionsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-health-score-definitions
notes:
  - SDK Method used are
    devices.Devices.update_health_score_definitions_v1,

  - Paths used are
    post /dna/intent/api/v1/healthScoreDefinitions/bulkUpdate,
  - It should be noted that this module is an alias of health_score_definitions_bulk_update_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.health_score_definitions_bulk_update:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: '{{my_headers | from_json}}'
    payload:
    - id: string
      includeForOverallHealth: true
      synchronizeToIssueThreshold: true
      thresholdValue: 0

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "name": "string",
          "displayName": "string",
          "deviceFamily": "string",
          "description": "string",
          "includeForOverallHealth": true,
          "definitionStatus": "string",
          "thresholdValue": 0,
          "synchronizeToIssueThreshold": true,
          "lastModified": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2021 Andrii Bilorus <andrii.bilorus@gmail.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storageshare_info
version_added: "1.8.0"
short_description: Get Azure storage file share info
description:
    - Get facts for storage file share.
options:
    resource_group:
        description:
            - Name of the resource group to use.
        required: true
        type: str
    name:
        description:
            - Name of the storage file share.
        type: str
        required: false
    account_name:
        description:
            - Name of the parent storage account for the storage file share.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Andrii Bilorus (@ewscat)
'''

EXAMPLES = '''
---
- name: Get storage share details
  azure_rm_storageshare_info:
    name: testShare
    resource_group: myResourceGroup
    account_name: testStorageAccount


- name: Get all storage file shares in storage account
  azure_rm_storageshare:
    resource_group: myResourceGroup
    account_name: testStorageAccount
'''

RETURN = '''
state:
    description:
        - Facts about the current state of the storage file share
    returned: always
    type: complex
    contains:
            id:
                description:
                    - Resource ID of the storage file share
                sample: "/subscriptions/9e700857-1631-4d8a-aed5-908520ede375/resourceGroups/myResourceGroup/providers/Microsoft.Storage/
                         storageAccounts/mystorageaccount/fileServices/default/shares/myshare"
                returned: always
                type: str
            name:
                description:
                    - Name of the file share
                sample: myshare
                returned: always
                type: str
            type:
                description:
                    - The type of the resource
                sample: "Microsoft.Storage/storageAccounts/fileServices/shares"
                returned: always
                type: str
            etag:
                description:
                    - Resource Etag
                sample: "0x8D75E4BA3E275F1"
                returned: always
                type: str
            last_modified_time:
                description:
                    - Returns the date and time the file share was last modified
                sample: "2021-08-23T08:17:35+00:00"
                returned: always
                type: str
            metadata:
                description:
                    - A name-value pair to associate with the file share as metadata
                sample: '{"key1": "value1"}'
                returned: always
                type: dict
            share_quota:
                description:
                    - The maximum size of the file share, in gigabytes
                sample: 102400
                returned: always
                type: int
            access_tier:
                description:
                    - Access tier for specific file share
                sample: 'TransactionOptimized'
                returned: always
                type: str
            access_tier_change_time:
                description:
                    - Indicates the last modification time for file share access tier
                sample: "2021-08-23T08:17:35+00:00"
                returned: always
                type: str
            enabled_protocols:
                description:
                    - The authentication protocol that is used for the file share.
                sample: 'SMB'
                returned: always
                type: str
            root_squash:
                description:
                    - The property is for NFS share only. The default is NoRootSquash.
                sample: 'NoRootSquash'
                returned: always
                type: str
            version:
                description:
                    - The version of the file share
                returned: always
                type: str
            deleted:
                description:
                    - Indicates whether the share was deleted
                returned: always
                type: str
            deleted_time:
                description:
                    - The deleted time if the share was deleted
                returned: always
                type: str
            remaining_retention_days:
                description:
                    - Remaining retention days for share that
                returned: always
                type: str
            access_tier_status:
                description:
                    - Indicates if there is a pending transition for access tier
                returned: always
                type: str
            share_usage_bytes:
                description:
                    - The approximate size of the data stored on the share. Note that this value may not include
                      all recently created or recently resized files.
                returned: always
                type: int
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMStorageShareInfo(AzureRMModuleBase):
    '''
    Info class for an Azure RM Storage share resource
    '''

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str'),
            account_name=dict(type='str', required=True),
        )
        self.results = dict(
            changed=False,
            storageshares=list()
        )

        self.resource_group = None
        self.name = None
        self.account_name = None

        super(AzureRMStorageShareInfo, self).__init__(self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      supports_tags=False,
                                                      facts_module=True)

    def exec_module(self, **kwargs):
        '''
        Main module execution method
        '''

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['storageshares'] = self.get_share()
        else:
            self.results['storageshares'] = self.list_all()

        return self.results

    def get_share(self):
        '''
        Get the properties of the specified Azure Storage file share.
        :return: dict with properties of the storage file share
        '''
        storage_share = None
        try:
            storage_share = self.storage_client.file_shares.get(resource_group_name=self.resource_group,
                                                                account_name=self.account_name,
                                                                share_name=self.name)
            self.log("Response : {0}".format(storage_share))
        except ResourceNotFoundError as e:
            self.log("Did not find the storage share with name {0} : {1}".format(self.name, str(e)))
        return self.storage_share_to_dict(storage_share)

    def storage_share_to_dict(self, storage_share):
        '''
        Transform Azure RM Storage share object to dictionary
        :param storage_share: contains information about storage file share
        :type storage_share: FileShare
        :return: dict generated from storage_share
        '''
        return dict(
            id=storage_share.id,
            name=storage_share.name,
            type=storage_share.type,
            etag=storage_share.etag.replace('"', ''),
            last_modified_time=storage_share.last_modified_time,
            metadata=storage_share.metadata,
            share_quota=storage_share.share_quota,
            access_tier=storage_share.access_tier,
            access_tier_change_time=storage_share.access_tier_change_time,
            enabled_protocols=storage_share.enabled_protocols,
            root_squash=storage_share.root_squash,
            version=storage_share.version,
            deleted=storage_share.deleted,
            deleted_time=storage_share.deleted_time,
            remaining_retention_days=storage_share.remaining_retention_days,
            access_tier_status=storage_share.access_tier_status,
            share_usage_bytes=storage_share.share_usage_bytes
        ) if storage_share else None

    def list_all(self):
        '''
        Method calling the Azure SDK to create storage file share.
        :return: dict with description of the new storage file share
        '''
        '''
        Get the properties of the specified Azure Storage file share.
        :return: dict with properties of the storage file share
        '''
        all_items = None
        try:
            storage_shares = self.storage_client.file_shares.list(resource_group_name=self.resource_group,
                                                                  account_name=self.account_name)
            self.log("Response : {0}".format(storage_shares))
            all_items = [self.storage_share_to_dict(share) for share in storage_shares]
        except Exception as e:
            self.log("Did not find the storage file share : {0}".format(str(e)))
        return all_items


def main():
    AzureRMStorageShareInfo()


if __name__ == '__main__':
    main()
